@@ -35,7 +35,7 @@ pub trait Matrix {
     /// Transpose switches rows and columns
     fn transpose(&self) -> Self;
     /// Inverse of a Matrix NxN
-    fn inverse(self, delta: f32) -> Result<Self, Error>
+    fn inverse(&self, delta: f32) -> Result<Self, Error>
     where
         Self: Sized;
 }
@@ -129,7 +129,7 @@ impl Matrix for Matrix3x3 {
     /// let matrix_inverse = Matrix3x3::new_idx(-6.0, 3.6, 1.4, 5.0, -3.0, -1.0, -1.0,  0.8, 0.2);
     /// assert_eq!(matrix.inverse(0.001).unwrap(), matrix_inverse);
     /// ```
-    fn inverse(self, delta: f32) -> Result<Matrix3x3, Error> {
+    fn inverse(&self, delta: f32) -> Result<Matrix3x3, Error> {
         let modulus = self.modulus();
         if float_eq(modulus, 0f32, delta) {
             return Err(Error::SingularMatrixNotInversible);
@@ -143,11 +143,7 @@ impl Matrix for Matrix3x3 {
 impl Matrix3x3 {
     ///Creates a new Matrix3x3 from 3 vector3 rows
     pub fn new(r1: Vector3, r2: Vector3, r3: Vector3) -> Matrix3x3 {
-        Matrix3x3 {
-            r1: r1,
-            r2: r2,
-            r3: r3,
-        }
+        Matrix3x3 { r1, r2, r3 }
     }
 
     ///Creates a new Matrix3x3 from 9 indexed floats
@@ -284,8 +280,8 @@ impl Matrix for Matrix2x2 {
     /// let matrix_inverse = Matrix2x2::new_idx(0.6, -0.7, -0.2, 0.4);
     /// assert_eq!(matrix.inverse(0.001).unwrap(), matrix_inverse);
     /// ```
-    fn inverse(self, delta: f32) -> Result<Matrix2x2, Error> {
-        let det = self.clone().det();
+    fn inverse(&self, delta: f32) -> Result<Matrix2x2, Error> {
+        let det = self.det();
 
         if math_helper::float_eq(det, 0f32, delta) {
             Err(Error::NonZeroDeterminantMatrix)
@@ -301,7 +297,7 @@ impl Matrix for Matrix2x2 {
 impl Matrix2x2 {
     ///Creates a new Matrix2x2 from 2 vector2 rows
     pub fn new(r1: Vector2, r2: Vector2) -> Matrix2x2 {
-        Matrix2x2 { r1: r1, r2: r2 }
+        Matrix2x2 { r1, r2 }
     }
 
     ///Creates a new Matrix2x2 from 4 indexed floats
@@ -338,6 +334,31 @@ impl ops::Add for Matrix3x3 {
     }
 }
 
+impl ops::Add for &Matrix3x3 {
+    type Output = Matrix3x3;
+
+    ///Implements the &Matrix 3x3 '+' trait
+    fn add(self, new: &Matrix3x3) -> Matrix3x3 {
+        Matrix3x3::new(
+            Vector3::new(
+                self.r1.x + new.r1.x,
+                self.r1.y + new.r1.y,
+                self.r1.z + new.r1.z,
+            ),
+            Vector3::new(
+                self.r2.x + new.r2.x,
+                self.r2.y + new.r2.y,
+                self.r2.z + new.r2.z,
+            ),
+            Vector3::new(
+                self.r3.x + new.r3.x,
+                self.r3.y + new.r3.y,
+                self.r3.z + new.r3.z,
+            ),
+        )
+    }
+}
+
 impl ops::Mul<Vector3> for Matrix3x3 {
     type Output = Vector3;
 
@@ -345,9 +366,37 @@ impl ops::Mul<Vector3> for Matrix3x3 {
     /// The order should be matrix * vector becaus of 3x3 * 3x1 = 3x1
     fn mul(self, vec: Vector3) -> Vector3 {
         Vector3 {
-            x: self.r1 * vec.clone(),
-            y: self.r2 * vec.clone(),
-            z: self.r3 * vec.clone(),
+            x: &self.r1 * &vec,
+            y: &self.r2 * &vec,
+            z: &self.r3 * &vec,
+        }
+    }
+}
+
+impl ops::Mul<&Vector3> for Matrix3x3 {
+    type Output = Vector3;
+
+    ///Implements the transform matrix of a vector 3 into another vector 3.
+    /// The order should be matrix * vector becaus of 3x3 * 3x1 = 3x1
+    fn mul(self, vec: &Vector3) -> Vector3 {
+        Vector3 {
+            x: &self.r1 * vec,
+            y: &self.r2 * vec,
+            z: &self.r3 * vec,
+        }
+    }
+}
+
+impl ops::Mul<&Vector3> for &Matrix3x3 {
+    type Output = Vector3;
+
+    ///Implements the transform matrix of a vector 3 into another vector 3.
+    /// The order should be matrix * vector becaus of 3x3 * 3x1 = 3x1
+    fn mul(self, vec: &Vector3) -> Vector3 {
+        Vector3 {
+            x: &self.r1 * vec,
+            y: &self.r2 * vec,
+            z: &self.r3 * vec,
         }
     }
 }
@@ -365,6 +414,19 @@ impl ops::Mul<f32> for Matrix3x3 {
     }
 }
 
+impl ops::Mul<f32> for &Matrix3x3 {
+    type Output = Matrix3x3;
+
+    ///Implements the &Matrix 3x3 '*' trait for `&Matrix3x3 * f32` so that `(identity * value).det() == valueˆ3`.
+    fn mul(self, value: f32) -> Matrix3x3 {
+        Matrix3x3::new(
+            Vector3::new(self.r1.x * value, self.r1.y * value, self.r1.z * value),
+            Vector3::new(self.r2.x * value, self.r2.y * value, self.r2.z * value),
+            Vector3::new(self.r3.x * value, self.r3.y * value, self.r3.z * value),
+        )
+    }
+}
+
 impl ops::Mul<Matrix3x3> for f32 {
     type Output = Matrix3x3;
 
@@ -378,6 +440,19 @@ impl ops::Mul<Matrix3x3> for f32 {
     }
 }
 
+impl ops::Mul<&Matrix3x3> for f32 {
+    type Output = Matrix3x3;
+
+    ///Implements the &Matrix 3x3 '*' trait for `&Matrix3x3 * f32` so that `(value * identity).det() == valueˆ3` and `ßM == Mß`˜.
+    fn mul(self, m: &Matrix3x3) -> Matrix3x3 {
+        Matrix3x3::new(
+            Vector3::new(m.r1.x * self, m.r1.y * self, m.r1.z * self),
+            Vector3::new(m.r2.x * self, m.r2.y * self, m.r2.z * self),
+            Vector3::new(m.r3.x * self, m.r3.y * self, m.r3.z * self),
+        )
+    }
+}
+
 impl ops::Div<f32> for Matrix3x3 {
     type Output = Matrix3x3;
 
@@ -391,6 +466,19 @@ impl ops::Div<f32> for Matrix3x3 {
     }
 }
 
+impl ops::Div<f32> for &Matrix3x3 {
+    type Output = Matrix3x3;
+
+    ///Implements the &Matrix 3x3 '/' trait for `&Matrix3x3 / f32` so that `(identity / value).det() == valueˆ3`.
+    fn div(self, value: f32) -> Matrix3x3 {
+        Matrix3x3::new(
+            Vector3::new(self.r1.x / value, self.r1.y / value, self.r1.z / value),
+            Vector3::new(self.r2.x / value, self.r2.y / value, self.r2.z / value),
+            Vector3::new(self.r3.x / value, self.r3.y / value, self.r3.z / value),
+        )
+    }
+}
+
 impl ops::Add for Matrix2x2 {
     type Output = Matrix2x2;
 
@@ -403,6 +491,18 @@ impl ops::Add for Matrix2x2 {
     }
 }
 
+impl ops::Add for &Matrix2x2 {
+    type Output = Matrix2x2;
+
+    ///Implements the &Matrix 2x2 '+' trait
+    fn add(self, new: &Matrix2x2) -> Matrix2x2 {
+        Matrix2x2::new(
+            Vector2::new(self.r1.x + new.r1.x, self.r1.y + new.r1.y),
+            Vector2::new(self.r2.x + new.r2.x, self.r2.y + new.r2.y),
+        )
+    }
+}
+
 impl ops::Mul<f32> for Matrix2x2 {
     type Output = Matrix2x2;
 
@@ -415,6 +515,18 @@ impl ops::Mul<f32> for Matrix2x2 {
     }
 }
 
+impl ops::Mul<f32> for &Matrix2x2 {
+    type Output = Matrix2x2;
+
+    ///Implements the &Matrix 2x2 '*' trait for `&Matrix2x2 * f32` so that `(identity * value).det() == valueˆ2`.
+    fn mul(self, value: f32) -> Matrix2x2 {
+        Matrix2x2::new(
+            Vector2::new(self.r1.x * value, self.r1.y * value),
+            Vector2::new(self.r2.x * value, self.r2.y * value),
+        )
+    }
+}
+
 impl ops::Mul<Matrix2x2> for f32 {
     type Output = Matrix2x2;
 
@@ -427,6 +539,18 @@ impl ops::Mul<Matrix2x2> for f32 {
     }
 }
 
+impl ops::Mul<&Matrix2x2> for f32 {
+    type Output = Matrix2x2;
+
+    ///Implements the &Matrix 2x2 '*' trait for `&Matrix2x2 * f32` so that `(value * identity).det() == valueˆ2` and `ßM == Mß`˜.
+    fn mul(self, m: &Matrix2x2) -> Matrix2x2 {
+        Matrix2x2::new(
+            Vector2::new(m.r1.x * self, m.r1.y * self),
+            Vector2::new(m.r2.x * self, m.r2.y * self),
+        )
+    }
+}
+
 impl ops::Div<f32> for Matrix2x2 {
     type Output = Matrix2x2;
 
@@ -439,6 +563,18 @@ impl ops::Div<f32> for Matrix2x2 {
     }
 }
 
+impl ops::Div<f32> for &Matrix2x2 {
+    type Output = Matrix2x2;
+
+    ///Implements the Matrix 2x2 '/' trait for `&Matrix2x2 / f32` so that `(identity / value).det() == valueˆ2`.
+    fn div(self, value: f32) -> Matrix2x2 {
+        Matrix2x2::new(
+            Vector2::new(self.r1.x / value, self.r1.y / value),
+            Vector2::new(self.r2.x / value, self.r2.y / value),
+        )
+    }
+}
+
 impl ops::Mul<Vector2> for Matrix2x2 {
     type Output = Vector2;
 
@@ -446,8 +582,34 @@ impl ops::Mul<Vector2> for Matrix2x2 {
     /// The order should be matrix * vector becaus of 2x2 * 2x1 = 2x1
     fn mul(self, vec: Vector2) -> Vector2 {
         Vector2 {
-            x: self.r1 * vec.clone(),
-            y: self.r2 * vec.clone(),
+            x: &self.r1 * &vec,
+            y: &self.r2 * &vec,
+        }
+    }
+}
+
+impl ops::Mul<&Vector2> for Matrix2x2 {
+    type Output = Vector2;
+
+    ///Implements the transform matrix of a vector 2 into another vector 2.
+    /// The order should be matrix * &vector becaus of 2x2 * 2x1 = 2x1
+    fn mul(self, vec: &Vector2) -> Vector2 {
+        Vector2 {
+            x: &self.r1 * vec,
+            y: &self.r2 * vec,
+        }
+    }
+}
+
+impl ops::Mul<&Vector2> for &Matrix2x2 {
+    type Output = Vector2;
+
+    ///Implements the transform matrix of a vector 2 into another vector 2.
+    /// The order should be matrix * vector becaus of 2x2 * 2x1 = 2x1
+    fn mul(self, vec: &Vector2) -> Vector2 {
+        Vector2 {
+            x: &self.r1 * vec,
+            y: &self.r2 * vec,
         }
     }
 }
@@ -491,6 +653,24 @@ mod tests_matrix3x3 {
         assert_eq!(expected, actual);
     }
 
+    #[test]
+    fn vector_transform_by_matrix_by_ref() {
+        let vec = Vector3::new(1f32, 2f32, 3f32);
+        let matrix = Matrix3x3::new_idx(1f32, 2f32, 3f32, 4f32, 5f32, 6f32, 7f32, 8f32, 9f32);
+        let actual = &matrix * &vec;
+        let expected = Vector3::new(14f32, 32f32, 50f32);
+        assert_eq!(expected, actual);
+    }
+
+    #[test]
+    fn vector_transform_by_matrix_by_ref_vec_only() {
+        let vec = Vector3::new(1f32, 2f32, 3f32);
+        let matrix = Matrix3x3::new_idx(1f32, 2f32, 3f32, 4f32, 5f32, 6f32, 7f32, 8f32, 9f32);
+        let actual = matrix * &vec;
+        let expected = Vector3::new(14f32, 32f32, 50f32);
+        assert_eq!(expected, actual);
+    }
+
     #[test]
     fn seq_maxtrix_det() {
         let matrix = Matrix3x3::new_idx(1f32, 2f32, 3f32, 4f32, 5f32, 6f32, 7f32, 8f32, 5f32);
@@ -511,17 +691,35 @@ mod tests_matrix3x3 {
         assert_eq!(8f32, identities.det());
     }
 
+    #[test]
+    fn identity_plus_identity_has_det_8_by_ref() {
+        let identities = &Matrix3x3::IDENTITY() + &Matrix3x3::IDENTITY();
+        assert_eq!(8f32, identities.det());
+    }
+
     #[test]
     fn det_of_identity_times_3_is_27() {
         let identity_3 = Matrix3x3::IDENTITY() * 3f32;
         assert_eq!(27f32, identity_3.det());
     }
 
+    #[test]
+    fn det_of_identity_times_3_is_27_by_ref() {
+        let identity_3 = &Matrix3x3::IDENTITY() * 3f32;
+        assert_eq!(27f32, identity_3.det());
+    }
+
     #[test]
     fn det_of_4_times_identity_is_64() {
         let identity_4 = 4f32 * Matrix3x3::IDENTITY();
         assert_eq!(64f32, identity_4.det());
     }
+
+    #[test]
+    fn det_of_4_times_identity_is_64_by_ref() {
+        let identity_4 = 4f32 * &Matrix3x3::IDENTITY();
+        assert_eq!(64f32, identity_4.det());
+    }
 }
 
 #[cfg(test)]
@@ -568,18 +766,36 @@ mod tests_matrix2x2 {
         assert_eq!(4f32, identities.det());
     }
 
+    #[test]
+    fn identity_plus_identity_has_det_8_by_ref() {
+        let identities = &Matrix2x2::IDENTITY() + &Matrix2x2::IDENTITY();
+        assert_eq!(4f32, identities.det());
+    }
+
     #[test]
     fn det_of_identity_times_3_is_9() {
         let identity_3 = Matrix2x2::IDENTITY() * 3f32;
         assert_eq!(9f32, identity_3.det());
     }
 
+    #[test]
+    fn det_of_identity_times_3_is_9_by_ref() {
+        let identity_3 = &Matrix2x2::IDENTITY() * 3f32;
+        assert_eq!(9f32, identity_3.det());
+    }
+
     #[test]
     fn det_of_4_times_identity_is_16() {
         let identity_4 = 4f32 * Matrix2x2::IDENTITY();
         assert_eq!(16f32, identity_4.det());
     }
 
+    #[test]
+    fn det_of_4_times_identity_is_16_by_ref() {
+        let identity_4 = 4f32 * &Matrix2x2::IDENTITY();
+        assert_eq!(16f32, identity_4.det());
+    }
+
     #[test]
     fn vector_transform_by_matrix() {
         let vec = Vector2::new(1f32, 2f32);
@@ -589,6 +805,24 @@ mod tests_matrix2x2 {
         assert_eq!(expected, actual);
     }
 
+    #[test]
+    fn vector_transform_by_matrix_by_ref() {
+        let vec = Vector2::new(1f32, 2f32);
+        let matrix = Matrix2x2::new_idx(1f32, 2f32, 3f32, 4f32);
+        let actual = &matrix * &vec;
+        let expected = Vector2::new(5f32, 11f32);
+        assert_eq!(expected, actual);
+    }
+
+    #[test]
+    fn vector_transform_by_matrix_by_ref_vec_only() {
+        let vec = Vector2::new(1f32, 2f32);
+        let matrix = Matrix2x2::new_idx(1f32, 2f32, 3f32, 4f32);
+        let actual = matrix * &vec;
+        let expected = Vector2::new(5f32, 11f32);
+        assert_eq!(expected, actual);
+    }
+
     #[test]
     #[should_panic]
     fn matrix_inverse_panic_for_det_0() {
@@ -713,4 +947,26 @@ mod test_arithmetic {
             Matrix3x3::new_idx(2f32, 0f32, 0f32, 0f32, 2f32, 0f32, 0f32, 0f32, 2f32)
         );
     }
+
+    #[test]
+    fn maxtrix2x2_by_ref() {
+        let matrix = Matrix2x2::IDENTITY();
+
+        assert_eq!(&matrix / 2f32, Matrix2x2::new_idx(0.5, 0.0, 0.0, 0.5));
+        assert_eq!(&matrix * 2f32, Matrix2x2::new_idx(2.0, 0.0, 0.0, 2.0));
+    }
+
+    #[test]
+    fn maxtrix3x3_by_ref() {
+        let matrix = Matrix3x3::IDENTITY();
+
+        assert_eq!(
+            &matrix / 2f32,
+            Matrix3x3::new_idx(0.5f32, 0f32, 0f32, 0f32, 0.5f32, 0f32, 0f32, 0f32, 0.5f32)
+        );
+        assert_eq!(
+            &matrix * 2f32,
+            Matrix3x3::new_idx(2f32, 0f32, 0f32, 0f32, 2f32, 0f32, 0f32, 0f32, 2f32)
+        );
+    }
 }
